from datetime import date, timedelta
from typing import Union


def get_date(session: dict) -> str:
    if "date" not in session:
        session["date"] = date.today().strftime("%d-%b-%Y")
    return session["date"]  # type: ignore[no-any-return]


def set_date(date_: Union[str, date], session: dict) -> None:
    session["date"] = date_ if isinstance(date_, str) else date_.strftime("%d-%b-%Y")


def get_start_date(session: dict) -> str:
    if "start" not in session:
        session["start"] = get_first_day(date.today()).strftime("%d-%b-%Y")
    return session["start"]  # type: ignore[no-any-return]


def get_finish_date(session: dict) -> str:
    if "finish" not in session:
        session["finish"] = get_last_day(date.today()).strftime("%d-%b-%Y")
    return session["finish"]  # type: ignore[no-any-return]


def set_period(start: Union[str, date], finish: Union[str, date], session: dict) -> None:
    if start is not None:
        session["start"] = start if isinstance(start, str) else start.strftime("%d-%b-%Y")
    if finish is not None:
        session["finish"] = finish if isinstance(finish, str) else finish.strftime("%d-%b-%Y")


def get_first_day(dt: date, d_years=0, d_months=0) -> date:
    # d_years, d_months are "deltas" to apply to dt
    y, m = dt.year + d_years, dt.month + d_months
    a, m = divmod(m - 1, 12)
    return date(y + a, m + 1, 1)


def get_last_day(dt: date) -> date:
    return get_first_day(dt, 0, 1) + timedelta(-1)
