import uuid

from datetime import date, datetime
from typing import List, Optional, Union

from brewman.schemas import to_camel
from brewman.schemas.attendance_type import AttendanceType
from pydantic import BaseModel, validator


class AttendanceItem(BaseModel):
    id_: uuid.UUID
    code: int
    name: str
    designation: str
    department: str
    attendance_type: AttendanceType
    prints: str
    hours_worked: str
    full_day: Optional[bool]

    class Config:
        alias_generator = to_camel


class Attendance(BaseModel):
    date_: Optional[date]
    body: List[AttendanceItem]

    @validator("date_", pre=True)
    def parse_date(cls, value: Union[date, str]) -> date:
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()

    class Config:
        anystr_strip_whitespace = True
        alias_generator = to_camel
        json_encoders = {date: lambda v: v.strftime("%d-%b-%Y")}
