from datetime import date, datetime
from typing import List, Optional, Union

from brewman.schemas import to_camel
from brewman.schemas.account import AccountLink
from brewman.schemas.attendance_type import AttendanceType
from pydantic import BaseModel, validator


class EmployeeAttendanceItem(BaseModel):
    date_: date
    attendance_type: AttendanceType
    prints: str
    hours_worked: str
    full_day: Optional[bool]

    @validator("date_", pre=True)
    def parse_date(cls, value: Union[date, str]) -> date:
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()

    class Config:
        anystr_strip_whitespace = True
        alias_generator = to_camel
        json_encoders = {date: lambda v: v.strftime("%d-%b-%Y")}


class EmployeeAttendance(BaseModel):
    start_date: Optional[date]
    finish_date: Optional[date]
    employee: Optional[AccountLink]
    body: List[EmployeeAttendanceItem]

    @validator("start_date", pre=True)
    def parse_start_date(cls, value: Union[date, str]) -> date:
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()

    @validator("finish_date", pre=True)
    def parse_finish_date(cls, value: Union[date, str]) -> date:
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()

    class Config:
        anystr_strip_whitespace = True
        alias_generator = to_camel
        json_encoders = {date: lambda v: v.strftime("%d-%b-%Y")}
