import uuid

from decimal import Decimal
from typing import Optional

from pydantic import BaseModel, Field

from . import to_camel


class StockKeepingUnit(BaseModel):
    id_: Optional[uuid.UUID]
    units: str = Field(..., min_length=1)
    fraction: Decimal = Field(ge=1, default=1)
    product_yield: Decimal = Field(gt=0, le=1, default=1)
    cost_price: Decimal = Field(ge=0, multiple_of=0.01, default=0)
    sale_price: Decimal = Field(ge=0, multiple_of=0.01, default=0)

    class Config:
        anystr_strip_whitespace = True
        alias_generator = to_camel
