import uuid

from datetime import datetime
from typing import List, Optional

from pydantic import Field
from pydantic.main import BaseModel

from . import to_camel
from .role import RoleItem


class UserIn(BaseModel):
    name: str = Field(..., min_length=1)
    password: str
    locked_out: bool
    roles: List[RoleItem]

    class Config:
        anystr_strip_whitespace = True
        alias_generator = to_camel


class User(UserIn):
    id_: uuid.UUID

    class Config:
        anystr_strip_whitespace = True
        alias_generator = to_camel


class UserList(BaseModel):
    id_: uuid.UUID
    name: str
    roles: List[str]
    last_device: str
    last_date: Optional[datetime]

    class Config:
        anystr_strip_whitespace = True
        alias_generator = to_camel
        json_encoders = {datetime: lambda v: v.strftime("%d-%b-%Y %H:%M")}


class UserToken(BaseModel):
    id_: uuid.UUID
    name: str
    locked_out: bool
    password: str
    permissions: List[str]


class UserBlank(UserIn):
    name: str

    class Config:
        anystr_strip_whitespace = True
        alias_generator = to_camel
