import { CommonModule } from '@angular/common';
import { NgModule } from '@angular/core';
import { RouterModule, Routes } from '@angular/router';

import { AuthGuard } from '../auth/auth-guard.service';
import { CostCentreListResolver } from '../cost-centre/cost-centre-list-resolver.service';

import { PeriodDetailComponent } from './period-detail/period-detail.component';
import { PeriodListResolver } from './period-list-resolver.service';
import { PeriodListComponent } from './period-list/period-list.component';
import { PeriodResolver } from './period-resolver.service';

const periodRoutes: Routes = [
  {
    path: '',
    component: PeriodListComponent,
    canActivate: [AuthGuard],
    data: {
      permission: 'Recipes',
    },
    resolve: {
      list: PeriodListResolver,
    },
  },
  {
    path: 'new',
    component: PeriodDetailComponent,
    canActivate: [AuthGuard],
    data: {
      permission: 'Recipes',
    },
    resolve: {
      item: PeriodResolver,
      costCentres: CostCentreListResolver,
    },
  },
  {
    path: ':id',
    component: PeriodDetailComponent,
    canActivate: [AuthGuard],
    data: {
      permission: 'Recipes',
    },
    resolve: {
      item: PeriodResolver,
      costCentres: CostCentreListResolver,
    },
  },
];

@NgModule({
  imports: [CommonModule, RouterModule.forChild(periodRoutes)],
  exports: [RouterModule],
  providers: [PeriodListResolver, PeriodResolver],
})
export class PeriodRoutingModule {}
